﻿Public Class BandsExtractor

    Friend FhtBuffer() As Single
    Friend BandsBuffer() As Single

    Private Const kdb As Double = 8.685889638
    Private Const kVolt As Single = 3.3

    Private m_ylog As Boolean
    Private m_dbMax As Single
    Private m_dbMin As Single
    Private m_vmax As Single
    Private m_vmin As Single

    Friend Sub ExtractBands(ByVal nBands As Int32, _
                             ByVal freqmin As Single, _
                             ByVal freqmax As Single, _
                             ByVal dbmin As Single, _
                             ByVal dbmax As Single, _
                             ByVal xlog As Boolean, _
                             ByVal ylog As Boolean, _
                             ByVal SampleFreq As Int32, _
                             ByVal speed As Single)
        '
        'Dim sw1 As Diagnostics.Stopwatch = New Diagnostics.Stopwatch : sw1.Start()
        '
        If FhtBuffer Is Nothing Then Return
        '
        m_ylog = ylog
        m_dbMax = dbmax
        m_dbMin = dbmin
        m_vmin = CSng(kVolt * Math.Exp(dbmin / kdb))
        m_vmax = CSng(kVolt * Math.Exp(dbmax / kdb))
        '
        If BandsBuffer Is Nothing OrElse BandsBuffer.Length <> nBands Then
            ReDim BandsBuffer(nBands - 1)
        End If
        ' ---------------------------------------------------------------------
        Dim nn As Int32 = FhtBuffer.Length * 2
        ' --------------------------------------------------------------------- X scale - log / lin
        Dim kf, kk As Single
        If xlog Then
            kf = freqmin * nn / SampleFreq
            kk = CSng(Math.Log(freqmax / freqmin) / nBands)
        Else
            kf = CSng(nn / SampleFreq)
            kk = (freqmax - freqmin) / nBands
        End If
        ' --------------------------------------------------------------------- vars
        Dim max As Single
        Dim index As Int32
        Dim oldindex As Int32
        If xlog Then
            oldindex = CInt(kf)
        Else
            oldindex = CInt(kf * freqmin)
        End If
        Dim oldband As Int32 = -1
        Dim val As Single
        Dim db As Single
        Dim oldVal As Single = 0
        ' ---------------------------------------------------------------------
        For band As Int32 = 0 To nBands - 1
            ' ----------------------------------------------------------------- X scale - log / lin
            If xlog Then
                index = CInt(-0.5 + kf * Math.Exp(kk * (band + 1)))
            Else
                index = CInt(-0.5 + kf * (freqmin + kk * (band + 1)))
            End If
            If oldindex < 0 Then oldindex = index
            If index > oldindex Then
                ' ------------------------------------------------------------- find max value in the band
                max = 0
                For j As Int32 = oldindex To index - 1
                    ' TODO ( *2 ) ???
                    max = Math.Max(FhtBuffer(j) * 2, max)
                Next
                ' ------------------------------------------------------------- find mid value in the band
                'max = 0
                'For j As Int32 = oldindex To index - 1
                '    max += FhtBuffer(j)
                'Next
                'max /= (index - oldindex)
                ' ------------------------------------------------------------- Y scale - log / lin
                If ylog Then
                    db = CSng(kdb * Math.Log(max))
                    val = (db - dbmin) / (dbmax - dbmin)
                Else
                    val = (max * kVolt - m_vmin) / (m_vmax - m_vmin)
                End If
                ' ------------------------------------------------------------- limit from 0 to 1
                If val < 0 Then val = 0
                If val > 1 Then val = 1
                ' ------------------------------------------------------------- create a smoot interpolation
                If oldband >= 0 Then
                    Dim k As Single = (val - oldVal) / (band - oldband)
                    For j As Int32 = oldband + 1 To band
                        val = oldVal + (j - oldband) * k
                        SetBandsBufferValue(val, j, speed)
                    Next
                Else
                    ' --------------------------------------------------------- first band
                    For j As Int32 = oldband + 1 To band
                        SetBandsBufferValue(val, j, speed)
                    Next
                End If
                oldband = band
                oldindex = index
                oldVal = val
            End If
            ' ----------------------------------------------------------------- last band 
            If band = nBands - 1 Then
                For j As Int32 = oldband + 1 To band
                    SetBandsBufferValue(val, j, speed)
                Next
            End If
        Next
        '
        'ReportString = "ExtractBands time: " & (sw1.Elapsed.TotalMilliseconds * 1000).ToString("0") & " uS"
    End Sub

    Private Sub SetBandsBufferValue(ByVal val As Single, ByVal j As Int32, ByVal speed As Single)
        If val > BandsBuffer(j) Then
            speed *= 10
            If speed > 1 Then speed = 1
            If speed < 0.1 Then speed = 0.1
        End If
        BandsBuffer(j) += (val - BandsBuffer(j)) * speed
    End Sub

    Friend Function GetBandValueInVoltOrDecibel(ByVal index As Int32) As Single
        Dim v As Single = BandsBuffer(index)
        If m_ylog Then
            Return v * (m_dbMax - m_dbMin) + m_dbMin
        Else
            'Val = (max * kVolt - vmin) / (vmax - vmin)
            Return (v * (m_vmax - m_vmin) + m_vmin) '/ kVolt
        End If
    End Function


End Class
