﻿Class DisplayScope

    Private Const kVolt As Single = 200

    Private m_mvDiv As Single = 500
    Private m_msDiv As Single = 100
    Private m_neg As Boolean
    Private m_trig As Boolean

    Private textFont As Font = New Font("Tahoma", 8)
    Private BackColor As Color = Color.FromArgb(30, 90, 120)
    Private scalePen As Pen = New Pen(Color.FromArgb(45, 58, 70))
    Private tracePen As Pen = New Pen(Color.FromArgb(240, 255, 255))

    Friend Sub Initialize(ByVal pbox As PictureBox)
        CloseAll()

    End Sub

    Friend Sub CloseAll()

    End Sub

    Friend Sub SetVoltage(ByVal mvDiv As Single)
        m_mvDiv = mvDiv
    End Sub

    Friend Sub SetTime(ByVal msDiv As Single)
        m_msDiv = msDiv
    End Sub

    Friend Sub SetNegative(ByVal negative As Boolean)
        m_neg = negative
    End Sub

    Friend Sub SetTrig(ByVal trigger As Boolean)
        m_trig = trigger
    End Sub

    Friend Sub Update(ByVal pbox As PictureBox)

        'Dim sw1 As Diagnostics.Stopwatch = New Diagnostics.Stopwatch : sw1.Start()
        ' ----------------------------------------------------------------- 
        InitPictureboxImage(pbox)
        ' ----------------------------------------------------------------- 
        Dim i As Int32
        Dim x As Single
        Dim y As Single
        Dim oldx As Int32
        Dim oldy As Single
        Dim h As Int32 = pbox.Image.Height
        Dim wpix As Int32 = pbox.Image.Width
        Dim h2 As Single = h / 2.0F
        Dim kmv As Single = h * kVolt / m_mvDiv
        Dim kw As Single = SamplesPerSec / 100.0F * m_msDiv / wpix
        Dim preTrigSamples As Int32
        Dim FifoReadIndex As Int32
        ' ----------------------------------------------------------------- 
        Dim g As Graphics = Graphics.FromImage(pbox.Image)
        g.Clear(BackColor)
        ' ----------------------------------------------------------------- draw scale
        For i = 0 To 9
            x = wpix * i \ 10
            g.DrawLine(scalePen, x, 0, x, h)
        Next
        For i = 0 To 7
            y = h * i \ 8
            g.DrawLine(scalePen, 0, y, wpix, y)
        Next

        Dim v As Single

        ' -------------------------------------------------------------- prepare the FIFO read position
        FifoReadIndex = CInt(Fifo1.GetWriteIndex - 1)
        If m_trig Then
            preTrigSamples = Math.Min(3 * CInt(kw * wpix), FIFO_Lenght \ 2)
            ' -------------------------------------------------------------- adaptive trigger
            Static TrigMin As Single
            Static TrigMax As Single
            Dim min As Single = Single.MaxValue
            Dim max As Single = Single.MinValue
            Dim TrigState As Int32 = 0
            For ix As Int32 = FifoReadIndex To FifoReadIndex - preTrigSamples Step -1
                v = Fifo1.GetElement(ix)
                If m_neg Then v = -v
                If v > max Then max = v
                If v < min Then min = v
                Select Case TrigState
                    Case 0
                        If v < TrigMin Then
                            TrigState = 1
                        End If
                    Case 1
                        If v > TrigMax Then
                            TrigState = 2
                            FifoReadIndex = ix
                        End If
                End Select
            Next
            ' ----------------------------------------------------------------- TrigMax and TrigMin
            Dim margin As Single = (max - min) * 0.25F ' was 0.1F
            TrigMax = max - margin
            TrigMin = min + margin
        End If
        ' ----------------------------------------------------------------- FifoIndex for the left of the window
        FifoReadIndex = FifoReadIndex - CInt(wpix * kw)


        Dim continueCounter As Int32
        ' ----------------------------------------------------------------- wave
        Dim deltaindex As Int32
        Static olddeltaindex As Int32
        g.SmoothingMode = Drawing2D.SmoothingMode.HighQuality
        oldx = -1
        oldy = 0
        For ix As Int32 = 0 To wpix - 1

            deltaindex = CInt(ix * kw)
            ' -------------------------------------------------------------- plot only if new sample
            If deltaindex = olddeltaindex Then
                Continue For
            End If
            olddeltaindex = deltaindex

            v = Fifo1.GetElement(FifoReadIndex + deltaindex) * 2
            If m_neg Then v = -v
            y = h2 - kmv * v

            ' -------------------------------------------------------------- plot only if y changes
            If Math.Abs(oldy - y) < 0.5 Then
                continueCounter += 1
                If continueCounter <= 1 Then Continue For
            End If
            continueCounter = 0

            If oldx >= 0 Then
                g.DrawLine(tracePen, oldx, oldy, ix, y)
                'g.DrawLine(tracePen, ix + 2, y, ix + 3, y)
            End If
            oldx = ix
            oldy = y
        Next
        ' ----------------------------------------------------------------- text
        g.DrawString(m_mvDiv.ToString() & " mV", textFont, Brushes.Azure, 5, h - 20)
        If m_msDiv >= 1 Then
            g.DrawString(m_msDiv.ToString() & " mS", textFont, Brushes.Azure, wpix - 46, h - 20)
        Else
            g.DrawString((m_msDiv * 1000).ToString() & " uS", textFont, Brushes.Azure, wpix - 45, h - 20)
        End If

        If HarmonicsExtractor1.HarmonicDistortion > 100 Then
            g.DrawString("THD over 100%", _
                         textFont, Brushes.Azure, 2, 2)
        Else
            g.DrawString("THD = " + _
                         HarmonicsExtractor1.HarmonicDistortion.ToString("0.00", GCI) + " %", _
                         textFont, Brushes.Azure, 2, 2)
        End If

        ' ----------------------------------------------------------------- show
        pbox.Invalidate()
        'ReportString = "Scope time: " & (sw1.Elapsed.TotalMilliseconds * 1000).ToString("0") & " uS"
    End Sub

End Class

